from pycomedi.single_aio import AO

class x_piezoError (Exception) :
    pass

class x_piezo :
    def __init__(self, xp_chan=1) :
        self.verbose = False
        self.xpSensitivity = 34.0 # nm/Volt
        self.gain = 20 # Vpiezo / Voutput

        self.AO = AO(chan=(xp_chan,))
        self.AO.close()
	#self.AO = sngAO.sngAOobj([xp_chan], -10, 10)

	self.curOut = [self.nm2out(0)]*1
	self.xpMax = self.nm2out(2000) # limits at 2 microns
	self.xpMin = self.nm2out(-2000) # and -2 microns

	self.wanderStep = self.nm2out(5) - self.nm2out(0)
	# define default value changed callbacks
	self.setExternalXPiezo = None
    def curPos(self) :
        return self.curOut[0]
    def nm2out(self, nm) :
        # nm / (nm/Vp * Vp/Vout) = Vout
        # where Vout is Volts output by the DAQ card
        # and Vp is Volts applied across the piezo
        return self.phys_to_out(nm / (self.xpSensitivity * self.gain))
    def out2nm(self, out) :
        return self.out_to_phys(out) * self.xpSensitivity * self.gain
    def out_to_phys(self, output) :
        return self.AO.comedi_to_phys(0, output)
    def phys_to_out(self, physical) :
        return self.AO.phys_to_comedi(0, physical)
    def pCurVals(self) :
        print "X piezo output  : %6d" % self.curPos()
    def jumpToPos(self, pos) :
        self._check_range(pos)
        if self.verbose :
            print "Jump X piezo to %d (%g nm)" % (pos, self.out2nm(pos))
	self.curOut[0] = pos
        self.AO.open()
	#self.AO.reserve()
	self.AO.write(self.curOut)
        self.AO.close()
	#self.AO.unreserve()
	if self.setExternalXPiezo != None :
	    self.setExternalXPiezo(self.out2nm(self.curPos()))
	return self.curPos()
    def wander(self) :
       	"""
	steps in one direction until it hits the boundary,
	then reverses and walks back the other way.
	"""
	newPos = self.curPos() + self.wanderStep
        if self.verbose :
            print "wander from %d to %d" % (self.curPos(), newPos)
	try :
	    self._check_range(newPos)
        except Exception :
	    # reached one limit, so turn around
	    self.wanderStep = -self.wanderStep
	    newPos += 2*self.wanderStep
	return self.jumpToPos(newPos)		
    def _check_range(self, pos) :
        if pos > self.xpMax :
	    raise x_piezoError, "X piezo pos = %d > %d = max" % (pos, self.xpMax)
	if pos < self.xpMin :
	    raise x_piezoError, "X piezo pos = %d < %d = min" % (pos, self.xpMin)

def test() :
    print "test x_piezo()"
    xp = x_piezo()
    xp.verbose = True
    xp.jumpToPos(xp.nm2out(0))
    xp.wander()
    xp.wander()
    xp.wander()
    xp.pCurVals()
    curPos = xp.curPos()
    curPosPhys = xp.out_to_phys(curPos)
    curPos2 = xp.phys_to_out(curPosPhys)
    if curPos != curPos2 :
        raise x_piezoError, "Conversion %d -> %g -> %d not round trip" % (curPos, curPosPhys, curPos2)
    print "success"

if __name__ == "__main__" :
    test()
